; LAML Manual page style

; COPYRIGHT (c)  Kurt Nrmark, Department of Computer Science, Aalborg University, Denmark.
; normark@cs.auc.dk,  http://www.cs.auc.dk/~normark/

; Warning: If schemdoc.scm is loaded after manual.scm, and if manual facility is used
; thereafter, problems occur. There is a conflict, yet unknown, between manual.scm and schemedoc.scm

; ---------------------------------------------------------------------------------------------------
; LOADING

(define (lib-load file-name)
  (load (string-append  the-library "/" file-name)))

(lib-load "general.scm")
(lib-load "cgi.scm")
(lib-load "html.scm")
(lib-load "html-v1.scm")
(lib-load "hex.scm")
(lib-load "time.scm")
(lib-load "file-read.scm")

; ---------------------------------------------------------------------------------------------------
; MAINLY PREAMBLE FUNCTION DEFINITIONS

(define laml-source-file-name-without-extension (source-filename-without-extension))

(define manual-title "manual")
(define manual-author '())
(define manual-abstract "")
(define home-url #f)
(define manual-master-index #f)

(define manual-name (if (not (equal? laml-source-file-name-without-extension ""))
                          laml-source-file-name-without-extension
                          "manual"))

(define (set-manual-name name)
  (set! manual-name name))

(define (set-manual-author . author-list)
  (set! manual-author author-list))

(define (set-manual-abstract abstract)
  (set! manual-abstract abstract))

(define (set-manual-title title)
  (set! manual-title title))

(define (set-home-url url)
  (set! home-url url))

(define (set-manual-master-index url)
 (set! manual-master-index url))


(define manual-page-list '()) ; the list in which we collect manual page descriptions

(define manual-background white)

(define language-preference 'english) ; redefinition

(define laml-manual-stuff #t)   ; true if making a manua for laml-stuff. Affects the header of the generated html file.

; ---------------------------------------------------------------------------------------------------
;  TOP LEVEL FUNCTION


; for manual sectioning:

;;; Programmatic support of manual generation. 
;;; The functions in the following are selected functions from the style
;;; manual.scm, which are useful when manuals are made from other programs.
;;; This part of the manual has been extracted automatically from the Scheme source file of the style.


; Define a manual page with a variety of different elements
(define (manual-page id . elements)
  (set! manual-page-list 
        (cons (cons (list 'kind "manual-page") elements) manual-page-list))
)

;; Given an entry (or page). Is it a manual page (as opposed to a manual section)
(define (manual-page? elements)
  (let ((res (assq 'kind elements)))
    (if res
        (equal? (cadr res) "manual-page")
        #f)))

;; Given an entry (or page). Is it a manual section (as opposed to a manual page)
(define (manual-section? elements)
  (let ((res (assq 'kind elements)))
    (if res
        (equal? (cadr res) "manual-section")
        #f)))

;; Define a manual section with title and body (plain text)
(define (manual-section . elements)
  (set! manual-page-list
	(cons (cons (list 'kind "manual-section") elements) manual-page-list)))


(define manual-first-sentence-in-string first-sentence-in-string)

; return a reference to another a-named place in the current document
(define (internal-ref anchor)
  (a-tag (this-reference anchor) anchor))

(define (this-reference ref)
  (string-append "#" ref))

; ---------------------------------------------------------------------------------------------------
; TAG GENERATION

(define (make-tag tagid)
  (lambda parameters (cons tagid parameters)))


(define title (make-tag 'title))
(define form (make-tag 'form))
(define pre-condition (make-tag 'pre-condition))
(define description (make-tag 'description))
(define parameters (make-tag 'parameters))
(define parameter (make-tag 'parameter))
(define example (make-tag 'example))
(define cross-references (make-tag 'cross-references))
(define reference (make-tag 'reference))
(define internal-references (make-tag 'internal-references))
(define comment (make-tag 'comment))
(define misc (make-tag 'misc))
(define returns (make-tag 'returns))
(define section-title (make-tag 'section-title))
(define section-body (make-tag 'section-body))
; no user level input for corresponding to library

; the mutual ordering among manual tags:
(define manual-element-order '(section-title section-body title library kind form description pre-condition parameters returns example cross-references misc comment))

; the list of possible and supported elements of a manual page. Manual section entries are not included in this list.
(define supported-manual-page-elements 
  '(title library kind form description pre-condition parameters example cross-references misc comment returns))

; ---------------------------------------------------------------------------------------------------
; DISPATCHING STUFF:

(define element-tag car)
(define element-contents cdr) 


(define (find-processor style kind)
  ; return function given the symbols style and kind
 (cond ((eq? style 'plain-manual-style)
        (cond ((eq? kind 'title) plain-manual-title)
              ((eq? kind 'form) plain-manual-form)
              ((eq? kind 'pre-condition) plain-manual-pre-condition)
              ((eq? kind 'description) plain-manual-description)
              ((eq? kind 'parameters) plain-manual-parameters)
              ((eq? kind 'parameter) plain-manual-parameter)
              ((eq? kind 'example) plain-manual-example)
              ((eq? kind 'cross-references) plain-manual-cross-references)
              ((eq? kind 'reference) plain-manual-reference)
              ((eq? kind 'internal-references) plain-manual-internal-references)
              ((eq? kind 'comment) plain-manual-comment)
              ((eq? kind 'misc) plain-manual-misc)
              ((eq? kind 'returns) plain-manual-returns)
              ((eq? kind 'kind) empty-element)
              ((eq? kind 'section-title) plain-manual-section-title)
              ((eq? kind 'section-body) plain-manual-section-body)
              ((eq? kind 'library) empty-element)
              (else (error (string-append (as-string kind) ": " "Cannot find tag in plain-manual-style")))))
       ((eq? style 'header-manual-style)
        (cond ((eq? kind 'title) header-manual-title)
              ((eq? kind 'form) header-manual-form)
              ((eq? kind 'pre-condition) empty-element)
              ((eq? kind 'description) header-manual-description)
              ((eq? kind 'parameters) empty-element)
              ((eq? kind 'parameter) empty-element)
              ((eq? kind 'example) empty-element)
              ((eq? kind 'cross-references) empty-element)
              ((eq? kind 'reference) empty-element)
              ((eq? kind 'internal-references) empty-element)
              ((eq? kind 'comment) empty-element)
              ((eq? kind 'misc) empty-element)
              ((eq? kind 'returns) empty-element)
              ((eq? kind 'kind) empty-element)
              ((eq? kind 'section-title) empty-element)
              ((eq? kind 'section-body) empty-element)
              ((eq? kind 'library) header-manual-library)
              (else (error (string-append (as-string kind) ": " "Cannot find tag in header-manual-style")))))
       ((eq? style 'toc-manual-style)
        (cond ((eq? kind 'title) empty-element)
              ((eq? kind 'form) empty-element)
              ((eq? kind 'pre-condition) empty-element)
              ((eq? kind 'description) empty-element)
              ((eq? kind 'parameters) empty-element)
              ((eq? kind 'parameter) empty-element)
              ((eq? kind 'example) empty-element)
              ((eq? kind 'cross-references) empty-element)
              ((eq? kind 'reference) empty-element)
              ((eq? kind 'internal-references) empty-element)
              ((eq? kind 'comment) empty-element)
              ((eq? kind 'misc) empty-element)
              ((eq? kind 'returns) empty-element)
              ((eq? kind 'kind) empty-element)
              ((eq? kind 'section-title) toc-manual-section-title)
              ((eq? kind 'section-body) empty-element)
              ((eq? kind 'library) empty-element)
              (else (error (string-append (as-string kind) ": " "Cannot find tag in toc-manual-style")))))
       (else (error "Unknown style"))))


(define (empty-element . pars)
  "")


(define (present-element style kind contents)
  (let ((processing-procedure (find-processor style kind)))
    (apply processing-procedure contents)))


; ---------------------------------------------------------------------------------------------------
; GENERATION OF PREDICATE BASED ON manual-element-order

; generate-leq is in the general library 

(define manual-element-ordering  (generate-leq manual-element-order element-tag))


; ---------------------------------------------------------------------------------------------------
; CENTRAL PRESENTATION FUNCTIONS:

; returns a list of html-markup
; elements is the documentation element (a list) of a single manual page
(define (format-manual-page elements style)
  (cond ((or (manual-section? elements) (manual-page? elements))  ; the two cases may be separated in the future, if needed
         (map
	  (lambda (e)
	    (present-element
	     style
	     (element-tag e)
	     (element-contents e)) 
	    )
	  (sort-list elements manual-element-ordering)))
        (else  (error "format-manual-page: unknown manual entry"))))


(define (string-concat string-list)
  (apply string-append string-list))

(define (line-breaker str)
  (con str (br)))

; manual is a list, such as manual-page-list
; thus manual is the whole manual struture.
; Returns a string a html markup
(define (format-manual manual style)
  (string-concat
    (map
      (lambda (manual-page)
        (con 
          (string-concat
           (map line-breaker
            (format-manual-page
              manual-page style)))
          (hr)
        )
      )
      manual)))

(define (present-author-info au)
 (let ((au1 (if (not (null? au)) (cons (copyright-owner (car au)) (cdr au)) au)))
   (h 3
      (con 
       (apply con
	      (map (lambda (e) (con e (horizontal-space 4))) au1))
       ))))

(define (present-abstract abstr)
 (let* ((offset 15)
        (width (- (accumulate-right + 0 manual-index-width-list) (* 2 offset))))
   (if (not (equal? abstr "")) (con (indent-pixels (+ 12 offset) (color-frame-width (em abstr) grey2 (- width offset))) (p)) "")  ))


;; This function makes the manual from collected information.
;; In this sense, this is the function which makes things happen.
;; If no first parameter is given, the manual is made from the collected information,
;; which is present in the variable manual-page-list.
;; If a parameter is given, the manual is made from this list of entries.
;; A second parameter it gives the absolute destination-directory.
;; A possible third parameter is the name of the source file. (This parameter is used for
;; informational purposes only).
;; This functions reverses the order of entries.
;; Take a look at one of the lsp files with manual information in order
;; to find out about the list structures of a manual-list.
;; A HTML file is written containing the formatted manual.
;; The HTML file is written to a particular destination-dirctory
;; (if this information is provided as the second parameter), else to the current directory.
(define (make-manual . par-list)
 (let* ((manual-page-list-1 (if (null? par-list) 
                                (reverse manual-page-list)  ; the global variable
                                (reverse (first par-list))  ; the first parameter - par-list is bound to a list of passed parameters...
                            ))
        (manual-page-list-2 (map clean-manual-page manual-page-list-1))   ; remove unkwown fields form manual pages
        (destination-dir (if (>= (length par-list) 2) (second par-list) ""))
        (source-file-name-information (if (>= (length par-list) 3) (third par-list) ""))
        (table-of-contents? (> (number-of-sections? manual-page-list-2) 0))
       )
  (write-text-file
   (page
      manual-title 
      (con
        (a-name "MANUAL-TOP")
        (if laml-manual-stuff 
            (left-middle-right-banner (when-generated) (string-append "Copyright " copyright " 2000, Kurt Nrmark")
                                      (laml-home-button 0 "laml-home.gif"))
            (left-middle-right-banner "" "" (em (when-generated))))
            
        (font-color blue (h 1 manual-title)) (p) 
        (present-author-info manual-author) (p)
        (if (and #f home-url (not (equal? home-url ""))) (con (a-tag home-url "Home") (space 3)) "")   ; no home page ref any more
        
        (if manual-master-index 
            (con (a-tag manual-master-index "Master index") (horizontal-space 15))
            "")

        (if (not (empty-string? source-file-name-information))
            (font-color grey (con "Source file: " source-file-name-information (br)))
            "")  
        (p)

        (present-abstract manual-abstract) (p)

        
        (if table-of-contents?
          (con 
            (a-name "MANUAL-TOC")
            (em "Table of Contents:")
            (indent-pixels 12 
              (con 
                (manual-table-of-contents manual-page-list-2 'toc-manual-style))) (p))
          "")

        (con
          (a-name "MANUAL-INDEX")
          (em "Alphabetic index:")
          (indent-pixels 12 
             (con 
                  (manual-index manual-page-list-2 'header-manual-style)))) (p)


        
        (format-manual manual-page-list-2 'plain-manual-style)
        (font 2 red (when-generated)) (br)
        (font 2 red end-remark)
        (vertical-space 25)  ; to allow for scrolling at the end of a manual page
      )
     )
   (string-append destination-dir (as-string manual-name) ".html"))

   ; Write the uncleaned manual page to a -.lsp file. This keeps specialized fields for other purposes.
   (if (<= (length par-list) 1)
       (write-lisp-form-manual manual-page-list-1)
       (write-lisp-form-manual manual-page-list-1 destination-dir))))

;; filter a manual page such that only known elements are processed.
(define (clean-manual-page page)
 (if (manual-page? page)
     (filter 
       (lambda (element) (memq (car element) supported-manual-page-elements)) 
       page)
     page))

    

;; A possible end remark added as the last line of the documentation in small font. Default is the empty string.
(define end-remark "") 


(define (manual-slice left-contribution right-contribution)
 ; return a horizontal slize of a manual contribution
 (table-1 0 '(12 140 12 "*")  (make-list 4 manual-background) 
      (list (list ""
                  (font-color red (b left-contribution))
                  ""
                  right-contribution)) 'top))

; ---------------------------------------------------------------------------------------------------
;;; Index functions. 
;;; The functions supporing the generation of a manual index.

;; The widths of the colums of the manual index.  A list of three integers
(define manual-index-width-list (list 140 300 400))

;; Return a manual index. Style should be the symbol header-manual-style
(define (manual-index manual style)
  (table-1 1 manual-index-width-list (list white white white)   
    (map
      (lambda (manual-page)
         (filter (lambda (e) (not (equal? e ""))) (format-manual-page manual-page style)))
      (sort-list 
         (filter manual-page? manual)  ; disregard manual sections for the index
          manual-index-order-leq?))))

;; The predicate ordering the manual entries for index sorting
(define (manual-index-order-leq? x y)
  (string<=? (cadr (assq 'title x)) (cadr (assq 'title y))))

;; Writes a manual structure (list of entries) to a file.
;; The filename is manual-name.lsp.
;; If the optional parameter dir is given write the lsp file in this directory.
;; Else write it in the current directory.
;; If provided, dir ends in a slash. 
;; The lsp file is useful for programmatic access to the entire body of manual information.
(define (write-lisp-form-manual manual-page-list . dir)
  (let* ((dir-1 (if (not (null? dir)) (first dir) ""))
         (lisp-form-manual-file-name (string-append dir-1 (as-string manual-name) ".lsp"))
        )
    (if (file-exists? lisp-form-manual-file-name)
	(delete-file lisp-form-manual-file-name))

    (with-output-to-file lisp-form-manual-file-name
			 (lambda () (write  manual-page-list)))))



; ---------------------------------------------------------------------------------------------------
; MANUAL PLAIN STYLE:

(define (plain-manual-title title-string)
  (con  (a-name title-string) (h 2 (font-color blue title-string))))

(define (plain-manual-form form-list)
  (manual-slice "Form"
    (box (kbd (as-string form-list)))))

(define (plain-manual-pre-condition precon)
 (manual-slice 
    "Preconditions"
     (box precon)))

(define (plain-manual-description descr)
 (manual-slice 
   "Description"
   (box (capitalize-string-nd (strip-initial-spaces descr)))))

(define (plain-manual-parameters . par-list)
 (manual-slice "Parameters"
   (table-3 0 '(140 5 "*")
    (map
     (lambda (par)
      (present-element 
        'plain-manual-style
        (element-tag par)
        (element-contents par)))
    par-list) 'top)))


(define (plain-manual-parameter par explanation)
  (list  par ""    explanation) ; fed into a table row
)

(define (plain-manual-example ex)
 (manual-slice 
   "Example" (box (pre (font-size 2 ex)))))

(define (plain-manual-cross-references . ref-list)
 (manual-slice "See also"
   (table-3 0 '(140 5 "*")
    (map
     (lambda (ref)
      (present-element 
        'plain-manual-style
        (element-tag ref)
        (element-contents ref)))
    ref-list) 'top)))


(define (plain-manual-reference ref anchor url)
  (list ref ""  (a-tag url anchor)) ; fed into a table row
)

(define (plain-manual-internal-references ref . ref-list)
  (list ref ""  (string-concat (map (lambda (r) (con (a-tag (string-append "#" r) r) (horizontal-space 4))) ref-list)))
)

(define (plain-manual-comment comment)
 (manual-slice 
   "Internal remark" (box (em (font-size 2 comment)))))


(define (plain-manual-misc m)
 (manual-slice 
    "Note"
     (box m)))

(define (plain-manual-returns r)
 (manual-slice 
    "Returns"
     (box r)))

(define (plain-manual-section-title title-string)
  (let ((section-id (next-section-id)) ; increments next-section-number!
        (number next-section-number)
       )
    (con  (vertical-space 1)
          (a-name section-id)
          (b (font-color blue (con (font-size 6 (con (as-string number) "." (horizontal-space 3))) (font-rise (upcase-string title-string) 5)))))))

(define (plain-manual-section-body body)
  body)


; ---------------------------------------------------------------------------------------------------
; HEADER PLAIN STYLE:


(define (header-manual-title title-string)
  (a-tag (string-append "#" title-string) title-string))

(define (header-manual-form form-list)
  (as-string form-list))

(define (header-manual-description descr)
 (capitalize-string-nd (strip-initial-spaces (manual-first-sentence-in-string descr))))

(define (header-manual-library library)
 (b library))

; ---------------------------------------------------------------------------------------------------
; TABLE OF CONTENTS STYLE:

(define (toc-manual-section-title title-string)
  title-string
 ; (a-tag (string-append "#" title-string) title-string)
 )


; ---------------------------------------------------------------------------------------------------

;;; Support of a master index. 
;;; A collection of indexes from several manual pages is called a master index.
;;; A master index is made manually from a separate LAML file which reads the saved .lsp files.
;;; The master index must be in the same directory as all the contributions.


;; Return the saved name.lsp manual info. Augment each entry with the library name and path information.
;; In that way we know for each manual entry where it comes from.
;; The second parameter is optional. If provided it gives the directory from which to read the lsp files.
;; The directory information is relative to the-library directory.
;; If it is not provided, read from the current directory.
(define (read-and-augment-lsp-file name . dir)
  (let ((dir-1 (if (not (null? dir)) (first dir) "")))
   (let ((res (file-read (string-append the-library dir-1 name ".lsp"))))
    (map 
      (lambda (entry) (cons (list 'path dir-1) (cons (list 'library name) entry)))
      res))))



;; The column widths of the master index. A list of 4 integers.
(define master-manual-index-width-list (list 140 70 300 400))

;; Return the master index given an augmented manual.
;; The augmentation is information about the library of each entry.
(define (manual-masterindex augmented-manual style)
  (table-1 1 master-manual-index-width-list (list white white white white)   
    (map
      masterindex-present-special
      (sort-list 
         (filter manual-page? augmented-manual)  ; disregard manual sections for the index
          manual-index-order-leq?))))

(define (masterindex-present-special entry)
  ; We use this special presentation because we need a non-local reference from the first column.
  ; Thus we do not use the nice style-based presentation, already prepared for the library entry
  ; this information cannot be made from the title alone.
  (let ((title (cadr (assq 'title entry)))
        (library (cadr (assq 'library entry)))
        (path (cadr (assq 'path entry)))
        (form (cadr (assq 'form entry)))
        (description (cadr (assq 'description entry))))
   (list (master-index-ref title library path) (b library) (as-string form) (manual-first-sentence-in-string description))))

(define (master-index-ref title library path)
  (a-tag (string-append path library ".html" "#" title) title))


;; Write a master index to the file manual-name.html.
;; The second parameter is optional. If provided it gives the directory of the resulting file. If not, write in the current directory.
;; A master index is a normal index, but in addition it contains a libray column
;; Normally the parameters all-entries are made by mapping read-and-augment-lsp-file over the saved lsp files with
;; manual information. Next these lists are append accumulated.
(define (make-master-index all-entries . dir)
  (let ((dir-1 (if (not (null? dir)) (first dir) "")))
   (write-text-file
    (page
       manual-title
       (con
        (left-middle-right-banner (when-generated) (string-append "Copyright " copyright " 2000, Kurt Nrmark") (laml-home-button 0 "laml-home.gif"))
        (font-color blue (h 1 manual-title)) (p) 
        (present-author-info manual-author) (p)
        (if (and #f home-url) (con (a-tag home-url "Home") (p)) "")
        (indent-pixels 12 
           (con (em "Alphabetic index:") 
           (manual-masterindex all-entries 'header-manual-style))) (p)
        (font 2 red (when-generated)) (br)
        (font 2 red end-remark)
       ))
     (string-append dir-1 (as-string manual-name) ".html"))))


;; Old
(define (manual-table-of-contents manual style)
 (let ((sections (filter manual-section? manual)))
  (ol
    (map2
      (lambda (manual-section section-number) 
        (let ((section-text  (car (format-manual-page manual-section style))))
          (a-tag (string-append "#" (section-id section-number)) section-text))
      )
      sections
      (number-interval 1 (length sections))))))


(define (manual-table-of-contents manual style)
 (let ((sections (filter manual-section? manual))
       (toc-width (accumulate-right + 0 manual-index-width-list)))
  (n-column-list 3
    (map2
      (lambda (manual-section section-number) 
        (let ((section-text  (car (format-manual-page manual-section style))))
          (a-tag
            (string-append "#" (section-id section-number))
            (font-size 2 (con (as-string section-number) ". " section-text))))
      )
      sections
      (number-interval 1 (length sections)))
    toc-width )))

(define (number-of-sections? manual)
  (length (filter manual-section? manual)))

; ---------------------------------------------------------------------------------------------------
; SECTION ID ADMINISTRATION

(define next-section-number 0)

; Return the next section id.
(define (next-section-id)
  (set! next-section-number (+ next-section-number 1))
  (string-append "SECTION" (as-string next-section-number)))

; Return section id of section number (an integer)
(define (section-id number)
  (string-append "SECTION" (as-string number)))

; ---------------------------------------------------------------------------------------------------

(laml-welcome)

